# frozen_string_literal: true

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Auto-generated by gapic-generator-ruby. DO NOT EDIT!


module Google
  module Api
    # Monitoring configuration of the service.
    #
    # The example below shows how to configure monitored resources and metrics
    # for monitoring. In the example, a monitored resource and two metrics are
    # defined. The `library.googleapis.com/book/returned_count` metric is sent
    # to both producer and consumer projects, whereas the
    # `library.googleapis.com/book/overdue_count` metric is only sent to the
    # consumer project.
    #
    #     monitored_resources:
    #     - type: library.googleapis.com/branch
    #       labels:
    #       - key: /city
    #         description: The city where the library branch is located in.
    #       - key: /name
    #         description: The name of the branch.
    #     metrics:
    #     - name: library.googleapis.com/book/returned_count
    #       metric_kind: DELTA
    #       value_type: INT64
    #       labels:
    #       - key: /customer_id
    #     - name: library.googleapis.com/book/overdue_count
    #       metric_kind: GAUGE
    #       value_type: INT64
    #       labels:
    #       - key: /customer_id
    #     monitoring:
    #       producer_destinations:
    #       - monitored_resource: library.googleapis.com/branch
    #         metrics:
    #         - library.googleapis.com/book/returned_count
    #       consumer_destinations:
    #       - monitored_resource: library.googleapis.com/branch
    #         metrics:
    #         - library.googleapis.com/book/returned_count
    #         - library.googleapis.com/book/overdue_count
    # @!attribute [rw] producer_destinations
    #   @return [::Array<::Google::Api::Monitoring::MonitoringDestination>]
    #     Monitoring configurations for sending metrics to the producer project.
    #     There can be multiple producer destinations, each one must have a
    #     different monitored resource type. A metric can be used in at most
    #     one producer destination.
    # @!attribute [rw] consumer_destinations
    #   @return [::Array<::Google::Api::Monitoring::MonitoringDestination>]
    #     Monitoring configurations for sending metrics to the consumer project.
    #     There can be multiple consumer destinations, each one must have a
    #     different monitored resource type. A metric can be used in at most
    #     one consumer destination.
    class Monitoring
      include ::Google::Protobuf::MessageExts
      extend ::Google::Protobuf::MessageExts::ClassMethods

      # Configuration of a specific monitoring destination (the producer project
      # or the consumer project).
      # @!attribute [rw] monitored_resource
      #   @return [::String]
      #     The monitored resource type. The type must be defined in
      #     {::Google::Api::Service#monitored_resources Service.monitored_resources} section.
      # @!attribute [rw] metrics
      #   @return [::Array<::String>]
      #     Names of the metrics to report to this monitoring destination.
      #     Each name must be defined in {::Google::Api::Service#metrics Service.metrics} section.
      class MonitoringDestination
        include ::Google::Protobuf::MessageExts
        extend ::Google::Protobuf::MessageExts::ClassMethods
      end
    end
  end
end
